#include "nrf_tone.h"
#include "nrf.h"
#include "nrf_gpiote.h"
#include "nrf_gpio.h"
#if(USE_WITH_SOFTDEVICE == 1)
#include "nrf_sdm.h"
#include "math.h"
#endif


static uint8_t	pin_number = 8;

static uint32_t  pwm_nbr_ticks;
static uint32_t  delay_nbr_loop = 10000;  // max 2^16
static uint32_t	 delay_cpt_limit = 0;
static uint32_t	 delay_cpt = 0;

static bool 			currently_playing = false;
static uint32_t 	playhead;

//MELODY
static uint16_t local_melody[MELODY_MAX_SIZE];
static float		local_beat[MELODY_MAX_SIZE];
static uint32_t local_size;
static uint8_t	local_tempo;


/** @brief Cette fonction retourne le nombre de ticks du timer qui correspond  la priode demande ( en uS)
 */
uint32_t period_to_ticks(uint32_t period, uint8_t prescal){
	
	float     master_clk = MASTER_CLOCK; // master clock
	float			ftimer;	 				//frequence timer en Hz
	float		 	tick_period ;		// dure de la priode en us
	float 		nbr_of_ticks;
	
	if ( prescal ==0){
		ftimer = master_clk ;
	}else{
		ftimer = (master_clk/(0x02<<(prescal-1)));	
	}
	
	tick_period = 1000000 / ftimer ;
	nbr_of_ticks = period/tick_period;
	return (uint32_t)nbr_of_ticks; 
}



/** @brief Function for initializing the Timer 2 peripheral.
 */
static void timer_init(void)
{
    // Start 16 MHz crystal oscillator .
    NRF_CLOCK->EVENTS_HFCLKSTARTED  = 0;
    NRF_CLOCK->TASKS_HFCLKSTART     = 1;

    // Wait for the external oscillator to start up.
    while (NRF_CLOCK->EVENTS_HFCLKSTARTED == 0) 
    {
        //Do nothing.
    }

    PWM_TIMER->MODE        	= TIMER_MODE_MODE_Timer;
		PWM_TIMER->BITMODE			= 0;
		PWM_TIMER->PRESCALER 		= PRESCAL ;
		
    // Clears the timer, sets it to 0.
    PWM_TIMER->TASKS_CLEAR = 1;
		
    // Interrupt setup.
    PWM_TIMER->INTENSET = TIMER_INTENSET_COMPARE0_Msk | TIMER_INTENSET_COMPARE1_Msk;
										 
		
}


/** @brief Function for initializing the GPIO Tasks/Events peripheral.
 */
static void gpiote_init(void)
{
    // Configure PWM_OUTPUT_PIN_NUMBER as an output. 
    nrf_gpio_cfg_output(pin_number);

    // Configure GPIOTE channel 0 to toggle the PWM pin state
    // @note Only one GPIOTE task can be connected to an output pin.
    nrf_gpiote_task_config(0, pin_number, NRF_GPIOTE_POLARITY_TOGGLE, NRF_GPIOTE_INITIAL_VALUE_LOW);
}




/** @brief Function for initializing the Programmable Peripheral Interconnect peripheral ( chanel 0)
 */
static void ppi_init(void)
{
uint32_t err_code;
uint32_t error;
	
#if(USE_WITH_SOFTDEVICE == 1)
				err_code = sd_ppi_channel_assign(0,&NRF_TIMER2->EVENTS_COMPARE[0], &NRF_GPIOTE->TASKS_OUT[0]);										// sd_ppi_channel_assign(ch_num, event_ptr, task_ptr);
        if( err_code != NRF_SUCCESS ){
					error = err_code;//erreur
				}
				
				err_code = sd_ppi_channel_enable_set(1 << 0);
				if( err_code != NRF_SUCCESS ){
					error = err_code;//erreur
				}
#else
        // Otherwise we configure the channel and return the channel number
        NRF_PPI->CH[0].EEP = (uint32_t)&NRF_TIMER2->EVENTS_COMPARE[0];
        NRF_PPI->CH[0].TEP = (uint32_t)&NRF_GPIOTE->TASKS_OUT[0];   
        
				// Enable PPI channels 0.
				NRF_PPI->CHEN = (PPI_CHEN_CH0_Enabled << PPI_CHEN_CH0_Pos); 
#endif
}


/** @brief Fonction appelle  la fin de chaque note
 */
void end_tone_handler(void){
	
	
	
}

	
	
	
/** @brief Function for initializing the Programmable Peripheral Interconnect peripheral ( chanel 0)
 */
void PWM_IRQHandler(void)
{
		// PWM Comparator
    if ((PWM_TIMER->EVENTS_COMPARE[0] != 0) && 
       ((PWM_TIMER->INTENSET & TIMER_INTENSET_COMPARE0_Msk) != 0))
    {
        // Sets the next CC1 value
        PWM_TIMER->EVENTS_COMPARE[0] = 0;
        PWM_TIMER->CC[0]             = PWM_TIMER->CC[0] + (pwm_nbr_ticks>>1) ;
			
    }
		
		if ((PWM_TIMER->EVENTS_COMPARE[1] != 0) && 
       ((PWM_TIMER->INTENSET & TIMER_INTENSET_COMPARE1_Msk) != 0))
    {
			delay_cpt++;
			PWM_TIMER->EVENTS_COMPARE[1] = 0;
			PWM_TIMER->CC[1] = PWM_TIMER->CC[1] + delay_nbr_loop;
			
			if(delay_cpt > delay_cpt_limit){
				PWM_TIMER->TASKS_STOP = 1;	
				PWM_TIMER->TASKS_CLEAR = 1;				
				delay_cpt = 0;	
				
				
				//Cas en cours de lecture
				if(playhead < local_size){
					
					//avancer la tte de lecture
					playhead++;
					//calcul de la dure de la note suivante
					uint32_t note_duration_ms;
					note_duration_ms = ((60000/local_tempo)*local_beat[playhead]);
					//jouer la note suivante
					tone(local_melody[playhead], note_duration_ms );
					
				}
				//Cas ou il n'y a plus de notes  lire
				else if(playhead >= local_size){
					// Remettre le flag lecture  0
					currently_playing = false;	
					
				}
	
			}
    }
}






/** @brief Function for initializing  the tone library
 */
uint32_t nrf_tone_init(){
		
	  gpiote_init();
	
    ppi_init();
	
	
    timer_init();
		
		#if(USE_WITH_SOFTDEVICE == 1)
			sd_nvic_SetPriority(TIMER2_IRQn, PWM_IRQ_PRIORITY | 0x01);
			sd_nvic_EnableIRQ(TIMER2_IRQn);
		#else
			NVIC_SetPriority(TIMER2_IRQn, PWM_IRQ_PRIORITY);
			NVIC_EnableIRQ(TIMER2_IRQn);
		#endif
	
	return 0 ;
}






/** @brief Fonction pour jouer une note de frequence freq pendant une dure duration (ms)
 */
uint32_t tone(uint32_t freq, uint32_t duration_ms){
	

	float period_ms;
	float ftimer;
	uint8_t prescaler = PRESCAL;
	

	period_ms = 1000/(float)freq;															// periode du signal pwm (ms)
	pwm_nbr_ticks = period_to_ticks(period_ms*1000,PRESCAL);  // nrb de ticks correpsondant la periode du signal pwm
	delay_cpt = 0;
	
	if ( prescaler == 0){
		ftimer = (float)MASTER_CLOCK ;
	}else{
		ftimer = ((float)MASTER_CLOCK/(0x02<<(prescaler-1)));	
	}

	// si la frequence est de 0, on dsactive le canal ppi
	if ( freq == 0 ){
		#if(USE_WITH_SOFTDEVICE == 1)
				sd_ppi_channel_enable_clr(1 << 0);
		#else
				NRF_PPI->CHEN = (PPI_CHEN_CH0_Disabled << PPI_CHEN_CH0_Pos); 
		#endif
		
	}else{
		#if(USE_WITH_SOFTDEVICE == 1)
				sd_ppi_channel_enable_set(1 << 0);
		#else
				NRF_PPI->CHEN = (PPI_CHEN_CH0_Enabled << PPI_CHEN_CH0_Pos); 
		#endif
	}
	
	//calcul du nombre de boucles
	delay_cpt_limit =  ((duration_ms*ftimer)/1000)/(float)delay_nbr_loop;   
		
	// initialisation comparateurs
	PWM_TIMER->CC[0] = pwm_nbr_ticks;
	PWM_TIMER->CC[1] = delay_nbr_loop;
	
	// Cas freq 0, desactivation du timer pwm
	if( freq == 0 ){ PWM_TIMER->EVENTS_COMPARE[0] = 1;   }
	
	

	// reset timer
	PWM_TIMER->TASKS_CLEAR = 1;
	
	// lancement du timer
  PWM_TIMER->TASKS_START = 1;
	
	return 0;
}



/** @brief Fonction pour jouer une mlodie. 
	arguments :
	melody[] contient les frquences des note  jouer
	beat[]		contient la dure des notes; 1 = noir, 2 = blanche, 0.5 = croche...
	size			Nombre d'lments contenus dans les tableaux
	tempo			Vitesse de la mlodie ( bpm)
 */

uint32_t play(uint32_t melody[], float beats[],uint32_t size, uint16_t tempo){
	
	//test if lib is already playong a melody
	if (currently_playing == true){
		return 8; // (NRF_ERROR_INVALID_STATE)
	}
	
	//copie des paramtres
	local_size  = size;
	local_tempo = tempo;
	for(int i = 0; i<size ; i++){
		local_melody[i] = melody[i];
		local_beat[i] = beats[i];
	}
	
	// Activation du flag de lecture
	currently_playing = true;
	
	// Positionnement de la tte de lecture
	playhead = 0;
	
	//calcul de la dure de la note
	uint32_t note_duration_ms;
	note_duration_ms = ((60000/tempo)*beats[0]);
	
	//lancement de la premire note
	tone(local_melody[0],note_duration_ms);
	
	return 0;
	
}
